# -*- coding: utf-8 -*-

# © 2020 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from datetime import datetime

from odoo import fields, models, api
from odoo.exceptions import UserError, Warning


class SaleOrderPcaFae(models.Model):
    _inherit = 'sale.order'

    order_pcafae_line_ids = fields.One2many(
        comodel_name='sale.order.pcafae.line',
        inverse_name='order_id',
        string='Pondération annuelle'
    )
    is_weighted = fields.Boolean(
        string='Ventilé',
        compute='_compute_is_weighted',
        store=True,
        default=False
    )

    # ------------------------------------------------------
    # Constrains function
    # ------------------------------------------------------
    @api.constrains('order_pcafae_line_ids')
    def _check_total_percent(self):
        total = sum(self.order_pcafae_line_ids.mapped('year_percent'))
        if self.order_pcafae_line_ids and total != 100.0:
            raise UserError("Le total des pourcentages doit être égal à 100 !")
        return True

    # ------------------------------------------------------
    # Computed fields
    # ------------------------------------------------------
    @api.multi
    @api.depends('order_pcafae_line_ids')
    def _compute_is_weighted(self):
        for so in self:
            if so.order_pcafae_line_ids:
                so.is_weighted = True
            else:
                so.is_weighted = False
    # ------------------------------------------------------
    # Override parent
    # ------------------------------------------------------
    @api.multi
    def action_invoice_create(self, grouped=False, final=False):
        """
        Hérite la fonction parente pour ajouter les lignes de ventilation
        sur la facture créée
        """
        res = super(SaleOrderPcaFae, self).action_invoice_create(grouped, final)
        if self.order_pcafae_line_ids:
            InvoicePcaFae = self.env['account.invoice.pcafae.line']
            for inv in res:
                for line in self.order_pcafae_line_ids:
                    InvoicePcaFae.create({
                        'year': line.year,
                        'invoice_id': inv,
                        'year_percent': line.year_percent,
                        'year_amount': line.year_amount,
                    })
        return res

    # ------------------------------------------------------
    # Override ORM
    # ------------------------------------------------------
    @api.model
    def create(self, vals):
        """
        Crée automatiquement une ligne PCA/FAE sur le devis
        """
        res = super(SaleOrderPcaFae, self).create(vals)
        company_current_year = datetime(
            datetime.now().year,
            self.env.user.company_id.fiscalyear_last_month,
            self.env.user.company_id.fiscalyear_last_day
        )
        date_order = datetime.strptime(res.create_date, '%Y-%m-%d %H:%M:%S')
        if date_order > company_current_year:
            year = str(date_order.year + 1)
        else:
            year = str(date_order.year)

        if not res.order_pcafae_line_ids:
            res.order_pcafae_line_ids.create({
                'order_id': res.id,
                'year_percent': 100.0,
                'year': year
            })
        return res
