# -*- coding: utf-8 -*-

# © 2020 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

import re

from datetime import datetime

from odoo import fields, models, api


class SaleOrderPcaFaeLines(models.Model):
    _name = 'sale.order.pcafae.line'
    _description = 'Lignes de previ commandes'
    _order = 'year, partner_id'

    year = fields.Char(
        string='Année Fiscale',
        default=str(datetime.now().year),
        required=True
    )
    order_id = fields.Many2one(
        comodel_name='sale.order',
        string='Commande',
        required=True
    )
    partner_id = fields.Many2one(
        comodel_name='res.partner',
        string='Client',
        related='order_id.partner_id',
        store=True
    )
    date = fields.Datetime(
        string='Date de commande',
        related='order_id.confirmation_date',
        store=True
    )
    state = fields.Selection(
        string='Status',
        related='order_id.state',
        store=True
    )
    company_id = fields.Many2one(
        comodel_name='res.company',
        string='Company',
        change_default=True, required=True, readonly=True,
        default=lambda self: self.env['res.company']._company_default_get(
            'sale.order.pcafae.line'))
    company_currency_id = fields.Many2one(
        comodel_name='res.currency',
        related='company_id.currency_id',
        string="Company Currency",
        readonly=True
    )
    year_percent = fields.Float('Pourcentage', default=0.0)
    year_amount = fields.Monetary(
        string='Montant HT',
        compute='_compute_year_amount',
        currency_field='company_currency_id',
        store=True
    )

    # ------------------------------------------------------
    # Contraintes SQL
    # ------------------------------------------------------
    # Unicité Année - Facture
    _sql_constraints = [
        ('year_unique',
         'unique(order_id, year)',
         "Il ne peut y avoir qu'une ligne année sur la commande"),
    ]

    # ------------------------------------------------------
    # Compute function
    # ------------------------------------------------------
    @api.depends('year_percent', 'order_id.amount_untaxed')
    @api.multi
    def _compute_year_amount(self):
        for line in self:
            line.year_amount = (0.01 * line.year_percent *
                                line.order_id.amount_untaxed)

    # ------------------------------------------------------
    # Onchange function
    # ------------------------------------------------------
    @api.onchange('year')
    def _onchange_year(self):
        valid_year = re.match('[2][0][1-4][0-9]', self.year)
        if not valid_year:
            return {
                'warning': {
                    'title': 'Erreur Valeur',
                    'message': "L'année n'a pas une valeur valide"
                }
            }